<?php

namespace MotaRaido\Core\Database;

if (!defined('MOTAFW')) 
{
    echo 'This file can only be called via the main index.php file, and not directly';
    exit();
}

use \MotaRaido\Core\Database\Connector;
use \Closure;

class Query
{
    private $dbh;

    public function __construct(Connector $dbh)
    {
        $this->dbh = $dbh;
        $this->dbh->setAttribute(Connector::ATTR_ERRMODE, Connector::ERRMODE_EXCEPTION);
        $this->dbh->forceRollback();
    }

    public function transaction(Closure $func) {
        $this->dbh->createTransaction($func);
    }
    
    public function select($tableName, $where)
    {
        $sql = $this->createSelectQuery($tableName, $where);
        $whereParams = $this->getWhereParameters($where);
        $stmp = $this->dbh->prepare($sql);
        $stmp->execute($whereParams);
        return $stmp->fetchAll();
    }
 
    public function insert($tableName, $values)
    {
        $sql = $this->createInsertQuery($tableName, $values);
        $stmp = $this->dbh->prepare($sql);
        $out = $stmp->execute($values);
        return $out;
    }
 
    public function update($tableName, $values, $where)
    {
        $sql = $this->createUpdateQuery($tableName, $values, $where);
        $whereParams = $this->getWhereParameters($where);
        $stmp = $this->dbh->prepare($sql);
        $out = $stmp->execute(array_merge($values, $whereParams));
        return $out;
    }
 
    public function delete($tableName, $where)
    {
        $sql = $this->createDeleteQuery($tableName, $where);
        $whereParams = $this->getWhereParameters($where);
        $stmp = $this->dbh->prepare($sql);
        $out = $stmp->execute($whereParams);
        return $out;
    }
 
    protected function getWhereParameters($where)
    {
        $whereParams = array();
        foreach ($where as $key => $value) {
            $whereParams[":W_{$key}"] = $value;
        }
        return $whereParams;
    }
 
    protected function createSelectQuery($table, $where)
    {
        return "SELECT * FROM " . $table . $this->createSqlWhere($where);
    }
 
    protected function createUpdateQuery($table, $values, $where)
    {
        $sqlValues = array();
        foreach (array_keys($values) as $key) {
            $sqlValues[] = "{$key} = :{$key}";
        }
        return "UPDATE {$table} SET " . implode(', ', $sqlValues) . $this->createSqlWhere($where);
    }
 
    protected function createInsertQuery($table, $values)
    {
        $sqlValues = array();
        foreach (array_keys($values) as $key) {
            $sqlValues[] = ":{$key}";
        }
        return "INSERT INTO {$table} (" . implode(', ', array_keys($values)) . ") VALUES (" . implode(', ', $sqlValues) . ")";
    }
 
    protected function createDeleteQuery($table, $where)
    {
        return "DELETE FROM {$table}" . $this->createSqlWhere($where);
    }
 
    protected function createSqlWhere($where)
    {
        if (count((array) $where) == 0) return null;
 
        $whereSql = array();
        foreach ($where as $key => $value) {
            $whereSql[] = "{$key} = :W_{$key}";
        }
        return ' WHERE ' . implode(' AND ', $whereSql);
    }
}